@@ -223,13 +223,30 @@ pub enum CommandValidationErrorType {
 pub fn command(value: &Command) -> Result<(), CommandValidationError> {
     let Command {
         description,
+        description_localizations,
         name,
+        name_localizations,
         kind,
         ..
     } = value;
 
     self::description(description)?;
 
+    if let Some(description_localizations) = description_localizations {
+        for description in description_localizations.values() {
+            self::description(description)?;
+        }
+    }
+
+    if let Some(name_localizations) = name_localizations {
+        for name in name_localizations.values() {
+            match kind {
+                CommandType::ChatInput => self::chat_input_name(name)?,
+                CommandType::User | CommandType::Message => self::name(name)?,
+            }
+        }
+    }
+
     match kind {
         CommandType::ChatInput => self::chat_input_name(name),
         CommandType::User | CommandType::Message => self::name(name),
@@ -490,6 +507,7 @@ mod tests {
     #![allow(clippy::non_ascii_literal)]
 
     use super::*;
+    use std::collections::HashMap;
     use twilight_model::{application::command::CommandType, id::Id};
 
     // This tests [`description`] and [`name`] by proxy.
@@ -497,12 +515,18 @@ mod tests {
     fn test_command() {
         let valid_command = Command {
             application_id: Some(Id::new(1)),
-            default_permission: None,
+            default_member_permissions: None,
+            dm_permission: None,
             description: "a".repeat(100),
+            description_localizations: Some(HashMap::from([(
+                "en-US".to_string(),
+                "a".repeat(100),
+            )])),
             guild_id: Some(Id::new(2)),
             id: Some(Id::new(3)),
             kind: CommandType::ChatInput,
             name: "b".repeat(32),
+            name_localizations: Some(HashMap::from([("en-US".to_string(), "b".repeat(32))])),
             options: Vec::new(),
             version: Id::new(4),
         };
